// サーバのアドレスとポート
var HOSTNAME = 'localhost';
var PORT = 8124;

// MySQL データベース名、ユーザー名、パスワード
var DBNAME = 'nodejs_url_shortener';
var DBUSER = 'root';
var DBPASSWD = null;

var sys = require('sys'),
    express = require('express'),
    ejs = require('ejs'),
    Client = require('mysql').Client,
    base62 = require('./base62');

// MySQLデータベースに接続しcallbackを呼び出す
function mysql(callback) {
    var client = new Client();
    client.database = DBNAME;
    client.user = DBUSER;
    client.password = DBPASSWD;

    client.connect(function(err) {
	if (err) {
	    throw err;
	}
	callback(client);
    });
}

var app = express.createServer();
app.use(express.bodyDecoder());
app.register('.ejs', ejs);

// ルート GET
app.get('/', function(req, res) {
    res.render('index.ejs');
});

// ルート POST
app.post('/', function(req, res) {
    // テンプレート変数
    var locals = {
	error: null,
	short_url: null
    };

    // パラメータをチェック
    if (!req.body.url) {
	locals.error = 'Missing url parameter';
    }  else if (req.body.url > 256) {
	locals.error = 'url parameter too long';
    }
    if (locals.error) {
	res.render('result.ejs', {
	    locals: locals
	});
	return;
    }

    // idを短縮URLに変換して出力
    function render_short_url(id) {
	locals.short_url = 'http://' + HOSTNAME;
	if (PORT != 80) {
	    locals.short_url += ':' + PORT;
	}
	locals.short_url += '/' + base62.int_to_base62_string(id);
	res.render('result.ejs', {
	    locals: locals
	});
    }

    // データベースに短縮URLを登録して表示
    mysql(function(client) {
	client.query(
	    'INSERT INTO shorten_urls (long_url) VALUES (?)',
	    [req.body.url],
	    function(err, results) {
		// キー重複は無視
		if (err && err.number != Client.ERROR_DUP_ENTRY) {
		    client.end();
		    throw err;
		}

		// インサート成功
		if (!err) {
		    render_short_url(results.insertId);
		    return;
		}

		// インサート失敗時はlong_urlをキーで検索する
		client.query(
		    'SELECT id FROM shorten_urls WHERE long_url = ?',
		    [req.body.url],
		    function(err, results, fields) {
			if (err) {
			    client.end();
			    throw err;
			}
			if (results.length == 0) {
			    client.end();
			    throw new Error('Something wrong');
			}
			client.end();
			render_short_url(results[0].id);
		    }
		);
	    }
	);
    });
});

// 短縮URLをリダイレクト
app.get(/^\/([0-9A-Z]{5,})$/, function(req, res) {
    mysql(function(client) {
	// idからurlを検索してリダイレクト
	client.query(
	    'SELECT long_url FROM shorten_urls WHERE id = ?',
	    [base62.base62_string_to_int(req.params[0])],
	    function(err, results, fields) {
		if (err) {
		    client.end();
		    throw err;
		}
		client.end();

		if (results.length == 0) {
		    // データが無い
		    res.send('Not Found', 404);
		} else {
		    res.redirect(results[0].long_url);
		}
	    }
	);
    });
});

app.listen(PORT, HOSTNAME);
